@@ -9,28 +9,28 @@
 # lost the next time ruby is built.
 
 module RbConfig
-  RUBY_VERSION.start_with?("3.4.") or
-    raise "ruby lib version (3.4.8) doesn't match executable version (#{RUBY_VERSION})"
+  RUBY_VERSION.start_with?("4.0.") or
+    raise "ruby lib version (4.0.0) doesn't match executable version (#{RUBY_VERSION})"
 
   # Ruby installed directory.
-  TOPDIR = File.dirname(__FILE__).chomp!("/lib/ruby/3.4/i386-openbsd")
+  TOPDIR = File.dirname(__FILE__).chomp!("/lib/ruby/4.0/i386-openbsd")
   # DESTDIR on make install.
   DESTDIR = '' unless defined? DESTDIR
   # The hash configurations stored.
   CONFIG = {}
   CONFIG["DESTDIR"] = DESTDIR
-  CONFIG["MAJOR"] = "3"
-  CONFIG["MINOR"] = "4"
-  CONFIG["TEENY"] = "8"
-  CONFIG["PATCHLEVEL"] = "72"
-  CONFIG["INSTALL"] = '/pobj/ruby-3.4.8/bin/install -c'
+  CONFIG["MAJOR"] = "4"
+  CONFIG["MINOR"] = "0"
+  CONFIG["TEENY"] = "0"
+  CONFIG["PATCHLEVEL"] = "0"
+  CONFIG["INSTALL"] = '/pobj/ruby-4.0.0/bin/install -c'
   CONFIG["EXEEXT"] = ""
   CONFIG["prefix"] = (TOPDIR || DESTDIR + "/usr/local")
-  CONFIG["ruby_install_name"] = "$(RUBY_BASE_NAME)34"
-  CONFIG["RUBY_INSTALL_NAME"] = "$(RUBY_BASE_NAME)34"
-  CONFIG["RUBY_SO_NAME"] = "ruby34"
+  CONFIG["ruby_install_name"] = "$(RUBY_BASE_NAME)40"
+  CONFIG["RUBY_INSTALL_NAME"] = "$(RUBY_BASE_NAME)40"
+  CONFIG["RUBY_SO_NAME"] = "ruby40"
   CONFIG["exec"] = "exec"
-  CONFIG["ruby_pc"] = "ruby-3.4.pc"
+  CONFIG["ruby_pc"] = "ruby-4.0.pc"
   CONFIG["CC_WRAPPER"] = ""
   CONFIG["PACKAGE"] = "ruby"
   CONFIG["BUILTIN_TRANSSRCS"] = " enc/trans/newline.c"
@@ -45,7 +45,7 @@ module RbConfig
   CONFIG["RUBY_SEARCH_PATH"] = ""
   CONFIG["UNIVERSAL_INTS"] = ""
   CONFIG["UNIVERSAL_ARCHNAMES"] = ""
-  CONFIG["configure_args"] = " '--enable-shared' '--program-suffix=34' '--with-soname=ruby34' '--with-ruby-version=3.4' '--with-mantype=doc' '--with-setjmp-type=_setjmp' '--enable-pthread' '--enable-ipv6' '--without-baseruby' '--without-bundled-libffi' '--disable-option-checking' '--with-compress-debug-sections=no' '--prefix=/usr/local' '--sysconfdir=/etc' '--mandir=/usr/local/man' '--infodir=/usr/local/info' '--localstatedir=/var' '--disable-silent-rules' '--disable-gtk-doc' 'CC=cc' 'CXX=c++' 'CFLAGS=-O2 -pipe' 'LDFLAGS=-L/usr/local/lib' 'CPPFLAGS=-DOPENSSL_NO_STATIC_ENGINE -I/usr/local/include' 'CXXFLAGS=-O2 -pipe'"
+  CONFIG["configure_args"] = " '--enable-shared' '--program-suffix=40' '--with-soname=ruby40' '--with-ruby-version=4.0' '--with-mantype=doc' '--with-setjmp-type=_setjmp' '--enable-pthread' '--enable-ipv6' '--without-baseruby' '--without-bundled-libffi' '--disable-option-checking' '--with-compress-debug-sections=no' '--prefix=/usr/local' '--sysconfdir=/etc' '--mandir=/usr/local/man' '--infodir=/usr/local/info' '--localstatedir=/var' '--disable-silent-rules' '--disable-gtk-doc' 'CC=cc' 'CXX=c++' 'CFLAGS=-O2 -pipe' 'LDFLAGS=-L/usr/local/lib' 'CPPFLAGS=-DOPENSSL_NO_STATIC_ENGINE -I/usr/local/include' 'CXXFLAGS=-O2 -pipe'"
   CONFIG["CONFIGURE"] = "configure"
   CONFIG["vendorarchdir"] = "$(vendorlibdir)/$(sitearch)"
   CONFIG["vendorlibdir"] = "$(vendordir)/$(ruby_version)"
@@ -55,7 +55,7 @@ module RbConfig
   CONFIG["sitedir"] = "$(rubylibprefix)/site_ruby"
   CONFIG["rubyarchdir"] = "$(rubylibdir)/$(arch)"
   CONFIG["rubylibdir"] = "$(rubylibprefix)/$(ruby_version)"
-  CONFIG["ruby_version"] = "3.4"
+  CONFIG["ruby_version"] = "4.0"
   CONFIG["sitearch"] = "$(arch)"
   CONFIG["arch"] = "i386-openbsd"
   CONFIG["sitearchincludedir"] = "$(includedir)/$(sitearch)"
@@ -64,7 +64,7 @@ module RbConfig
   CONFIG["archlibdir"] = "$(libdir)/$(arch)"
   CONFIG["libdirname"] = "libdir"
   CONFIG["RUBY_EXEC_PREFIX"] = "/usr/local"
-  CONFIG["RUBY_LIB_VERSION"] = "3.4"
+  CONFIG["RUBY_LIB_VERSION"] = "4.0"
   CONFIG["RUBY_LIB_VERSION_STYLE"] = ""
   CONFIG["RI_BASE_NAME"] = "ri"
   CONFIG["ridir"] = "$(datarootdir)/$(RI_BASE_NAME)"
@@ -105,11 +105,19 @@ module RbConfig
   CONFIG["NULLCMD"] = ":"
   CONFIG["ENABLE_DEBUG_ENV"] = ""
   CONFIG["DLNOBJ"] = "dln.o"
-  CONFIG["RJIT_SUPPORT"] = "no"
+  CONFIG["JIT_CARGO_SUPPORT"] = "no"
+  CONFIG["RLIB_DIR"] = ""
+  CONFIG["RUST_LIB"] = ""
+  CONFIG["JIT_OBJ"] = ""
+  CONFIG["ZJIT_OBJ"] = ""
+  CONFIG["ZJIT_LIBS"] = ""
+  CONFIG["ZJIT_SUPPORT"] = "no"
   CONFIG["YJIT_OBJ"] = ""
   CONFIG["YJIT_LIBS"] = ""
-  CONFIG["CARGO_BUILD_ARGS"] = ""
   CONFIG["YJIT_SUPPORT"] = "no"
+  CONFIG["CARGO_BUILD_ARGS"] = ""
+  CONFIG["JIT_RUST_FLAGS"] = "--crate-type=staticlib --cfg feature=\\\\\\\"stats_allocator\\\\\\\""
+  CONFIG["ZJIT_TEST_FEATURES"] = ""
   CONFIG["CARGO"] = ""
   CONFIG["RUSTC"] = "no"
   CONFIG["INSTALL_STATIC_LIBRARY"] = "no"
@@ -162,9 +170,9 @@ module RbConfig
   CONFIG["DOXYGEN"] = ""
   CONFIG["DOT"] = ""
   CONFIG["MKDIR_P"] = "mkdir -p"
-  CONFIG["INSTALL_DATA"] = "/pobj/ruby-3.4.8/bin/install -c -m 644"
-  CONFIG["INSTALL_SCRIPT"] = "/pobj/ruby-3.4.8/bin/install -c -m 755"
-  CONFIG["INSTALL_PROGRAM"] = "/pobj/ruby-3.4.8/bin/install -c -s -m 755"
+  CONFIG["INSTALL_DATA"] = "/pobj/ruby-4.0.0/bin/install -c -m 644"
+  CONFIG["INSTALL_SCRIPT"] = "/pobj/ruby-4.0.0/bin/install -c -m 755"
+  CONFIG["INSTALL_PROGRAM"] = "/pobj/ruby-4.0.0/bin/install -c -s -m 755"
   CONFIG["SET_MAKE"] = ""
   CONFIG["LN_S"] = "ln -s"
   CONFIG["DLLWRAP"] = ""
@@ -192,7 +200,7 @@ module RbConfig
   CONFIG["NM"] = "nm"
   CONFIG["LD"] = "ld"
   CONFIG["CXX"] = "c++"
-  CONFIG["AS"] = "gas"
+  CONFIG["AS"] = "as"
   CONFIG["AR"] = "ar"
   CONFIG["CC"] = "cc"
   CONFIG["wasmoptflags"] = ""
@@ -216,7 +224,7 @@ module RbConfig
   CONFIG["RUBY_PROGRAM_VERSION"] = "$(MAJOR).$(MINOR).$(TEENY)"
   CONFIG["RUBY_API_VERSION"] = "$(MAJOR).$(MINOR)"
   CONFIG["HAVE_GIT"] = "no"
-  CONFIG["GIT"] = "git"
+  CONFIG["GIT"] = "never-use"
   CONFIG["cxxflags"] = ""
   CONFIG["cppflags"] = ""
   CONFIG["cflags"] = "$(hardenflags) -fdeclspec  $(optflags) $(debugflags) $(warnflags)"
@@ -249,7 +257,7 @@ module RbConfig
   CONFIG["libexecdir"] = "$(exec_prefix)/libexec"
   CONFIG["sbindir"] = "$(exec_prefix)/sbin"
   CONFIG["bindir"] = "$(exec_prefix)/bin"
-  CONFIG["program_transform_name"] = "s&$$&34&;s&^&&"
+  CONFIG["program_transform_name"] = "s&$$&40&;s&^&&"
   CONFIG["exec_prefix"] = "$(prefix)"
   CONFIG["PACKAGE_URL"] = ""
   CONFIG["PACKAGE_BUGREPORT"] = ""
@@ -259,8 +267,8 @@ module RbConfig
   CONFIG["PACKAGE_NAME"] = ""
   CONFIG["PATH_SEPARATOR"] = ":"
   CONFIG["SHELL"] = "/bin/sh"
-  CONFIG["UNICODE_VERSION"] = "15.0.0"
-  CONFIG["UNICODE_EMOJI_VERSION"] = "15.0"
+  CONFIG["UNICODE_VERSION"] = "17.0.0"
+  CONFIG["UNICODE_EMOJI_VERSION"] = "17.0"
   CONFIG["platform"] = "$(arch)"
   CONFIG["archdir"] = "$(rubyarchdir)"
   CONFIG["topdir"] = File.dirname(__FILE__)
@@ -364,6 +372,7 @@ module RbConfig
     )
   end
 end
+# Non-nil if configured for cross compiling.
 CROSS_COMPILING = nil unless defined? CROSS_COMPILING
 install = '/usr/bin/install -c -o root -g bin'  
 c = RbConfig::CONFIG 
